import os
import subprocess
import yaml
import sys
import glob
import numpy as np
import argparse


class copy_from_esarchive():
    def __init__(self):
        USER=os.environ.get('USER')

        DEFAULT_ORIGIN = "/gpfs/archive/bsc32/esarchive/exp/CMIP6"
        DEFAULT_DESTINATION = f"/gpfs/scratch/bsc32/{USER}/climate_data/CMIP6/"

        # Set up argument parser
        parser = argparse.ArgumentParser(description="Script to copy data between directories connected in data transfer machines.")

        parser.add_argument(
            "--recipe",
            type=str,
            required=True,
            help=f"Mandatory --recipe flag\nThe path of the recipe must be added as argument:\npython copy_CMIP6_esarcvhie_to_gpfs.py --recipe=<recipe_path>"
        )

        parser.add_argument(
            "--origin",
            type=str,
            default=DEFAULT_ORIGIN,
            help=f"Optional --origin flag (default: {DEFAULT_ORIGIN})"
        )

        parser.add_argument(
            "--destination",
            type=str,
            default=DEFAULT_DESTINATION,
            help=f"Optional --desitnation flag (default: {DEFAULT_DESTINATION})"
        )

        # Parse the arguments
        args = parser.parse_args()

        self.recipe_path = args.recipe
        self.destination = args.destination
        self.origin = args.origin

    def main(self):
        with open(self.recipe_path, "r") as f:
            recipe = yaml.safe_load(f)
        exported_file = self.parse_recipe_data(recipe)
        self.call_rsync(exported_file)

    def parse_recipe_data(self,recipe):
        datasets = recipe["datasets"]
        exported_files = []
        for dataset in datasets:
            var_mips = set([f'{vars["short_name"]}-{vars["mip"]}' for diag in recipe["diagnostics"].values() for vars in diag["variables"].values()])
            for var_mip in var_mips:
                var, mip = var_mip.split("-")
                if isinstance(dataset["exp"], list):
                    for exp in dataset["exp"]:
                        if "(" in dataset["ensemble"] and ":" in dataset["ensemble"]:
                            ens_range=dataset["ensemble"].split("(")[-1].split(")")[0]
                            ensembles = [dataset["ensemble"].replace(f"({ens_range})", str(i)) for i in np.arange(int(ens_range.split(":")[0]), int(ens_range.split(":")[-1])+1)]
                        else:
                            ensembles = [dataset["ensemble"]]
                        for ensemble in ensembles:
                            cmor_path_version=f"{exp}/{dataset['dataset']}/*/*/{dataset['dataset']}/{exp}/{ensemble}/{mip}/{var}/{dataset['grid']}/*"
                            versions = glob.glob(os.path.join(self.origin, cmor_path_version))
                            if len(versions) == 0:
                                raise ValueError(f"Can't find the files for {dataset}")
                            version = sorted([i.split("/")[-1].split("v")[-1] for i in versions])[-1]
                            exported_files += glob.glob(os.path.join(self.origin, cmor_path_version[:-1]+f"*{version}", "*.nc"))
        return exported_files


    def call_rsync(self, exported_files):
        # Define the batch command as a list of arguments
        for file in exported_files:
            f = file.replace(self.origin, self.destination)
            if not os.path.isdir(os.path.dirname(f)):
                os.makedirs(os.path.dirname(f))
            if os.path.isfile(f):
                print(f"pass {f}")
                continue
            command = ["dtrsync", "-zaP", file, f]
            
            try:
                result = subprocess.run(command, capture_output=True, text=True)
                # Print stdout and stderr if needed
                print("stdout:", result.stdout)
                print("stderr:", result.stderr)
                print(f"File copied from {file} to {f} successfully.")
            except subprocess.CalledProcessError as e:
                print(f"Error occurred while copying file: {e}")
                print("stderr:", e.stderr) 

if __name__ == "__main__":
    copy_from_esarchive().main()